#include "ServiceManager.hpp"

// C++ system level
#include <vector>
#include <algorithm>
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>
// #include "SiliconTorch/NVSExplorer.hpp"

// qthing stuff
// #include <qthing>


static const char* TAG = "ServiceManager";


namespace SiliconTorch {

  namespace ServiceManager {


    void registerSiliconTorchServices(ServiceManager* mgr);


    ServiceManager::ServiceManager() {

      registerSiliconTorchServices(this);

      registrationLocked = true;


    }

    void ServiceManager::startEnabledServices() {
      std::vector<Service*> services;

      for (const auto& [key, value] : serviceMap)
        services.push_back(value);

      std::sort(services.begin(), services.end(), [](const Service* x, const Service* y) {
        return x->getStartOrder() < y->getStartOrder();
      });

      for (const auto& service : services)
        if (service->isEnabled()) service->start();
    }


    void ServiceManager::setServiceEnabled(const str& name, bool enabled) {
      if (serviceMap.count(name) > 0) {
        serviceMap.at(name)->setEnabled(enabled);
      } else {
        // Service not found
        // TODO: logging…?
      }
    }

    bool ServiceManager::getServiceEnabled(const str& name) {
      if (serviceMap.count(name) > 0) {
        return serviceMap.at(name)->isEnabled();
      } else {
        // Service not found
        // TODO: logging…?
        return false;
      }
    }


    void ServiceManager::registerService(Service* s) {
      if (!registrationLocked) {
        s->init();
        s->postInit();

        serviceMap[s->getName()] = s;

        ESP_LOGI(TAG, "Registered service[ %s ]", s->getName().c_str());
      } else {
        ESP_LOGW(TAG, "Service registration is locked");
      }
    }



  }
}
